@@ -30,13 +30,14 @@
 #include <algorithm>
 #include <cstring>
 #include <stdexcept>
+#include <climits>
 
 #include "compression/compressor.h"
 #include "interface/fstdefines.h"
 #include "interface/itypefactory.h"
 #include "interface/openmphelper.h"
 
-#include "LZ4/xxhash.h"
+#include "xxhash.h"
 #include <memory>
 
 enum COMPRESSION_ALGORITHM
@@ -101,126 +102,145 @@ public:
 
   /**
    * \brief Compress data with the 'LZ4' or 'ZSTD' compressor.
-   * \param blobSource source buffer to compress.
-   * \param blobLength Length of source buffer.
+   * \param blob_source source buffer to compress.
+   * \param blob_length Length of source buffer.
+   * \param hash if true, hashes are used in the compression result
    */
-  IBlobContainer* CompressBlob(unsigned char* blobSource, const unsigned long long blobLength, const bool hash = true) const
+  IBlobContainer* CompressBlob(unsigned char* blob_source, const unsigned long long blob_length, const bool hash = true) const
   {
-    int nrOfThreads = GetFstThreads();
+    int nr_of_threads = GetFstThreads();
 
     // Check for empty source
-    if (blobLength == 0)
+    if (blob_length == 0)
     {
       throw(std::runtime_error(FSTERROR_COMP_NO_DATA));
     }
 
-    // block size to use for compression has a lower bound for compression efficiency
-    const unsigned long long minBlock = std::max(static_cast<unsigned long long>(BLOCKSIZE), 1 + (blobLength - 1) / PREV_NR_OF_BLOCKS);
+    // block size to use for compression has a 16 kB lower bound for compression efficiency
+    const unsigned long long min_block = std::max(static_cast<unsigned long long>(BLOCKSIZE), 1ULL + (blob_length - 1ULL) / PREV_NR_OF_BLOCKS);
 
-    // and a higher bound for compressor compatability
-    const unsigned int blockSize = static_cast<unsigned int>(std::min(minBlock, static_cast<unsigned long long>(INT_MAX)));
+    // higher bound for compressor compatibility
+	// the factor 0.8 is to avoid compressed sizes larger than INT_MAX
+	// (assuming that 25% overhead will not occur in any compressor)
+    const unsigned long long block_size = static_cast<unsigned long long>(std::min(min_block, static_cast<unsigned long long>(0.8 * INT_MAX)));
 
-    const int nrOfBlocks = static_cast<int>(1 + (blobLength - 1) / blockSize);
-    nrOfThreads = std::min(nrOfThreads, nrOfBlocks);
+	// number of blocks in the compressed result
+    const unsigned long long nr_of_blocks = 1 + (blob_length - 1) / block_size;
 
-    const unsigned int maxCompressSize = this->compressor->CompressBufferSize(blockSize);
-    const unsigned int lastBlockSize = 1 + (blobLength - 1) % blockSize;
+	// do not use more threads than necessary or available
+	// if there are more block than threads, the default thread count is used
+	// each thread runs a single block batch
+    nr_of_threads = std::min(nr_of_threads, static_cast<int>(nr_of_blocks));
 
-    const unsigned long long bufSize = nrOfBlocks * maxCompressSize;
-    const float blocksPerThread = static_cast<float>(nrOfBlocks) / nrOfThreads;
+    const int max_compress_size = this->compressor->CompressBufferSize(block_size);
+    const unsigned long long last_block_size = 1 + (blob_length - 1) % block_size;
+
+	// this requires three times the original vector in memory
+	// perhaps a max number of blocks would be better (smaller buffer)
+    const unsigned long long buf_size = nr_of_blocks * static_cast<unsigned long long>(max_compress_size);
+    const double blocks_per_thread = static_cast<double>(nr_of_blocks) / nr_of_threads;
 
     // Compressed sizes
-    std::unique_ptr<unsigned long long[]> compSizesP(new unsigned long long[nrOfBlocks + 1]);
-    unsigned long long* compSizes = compSizesP.get();
+	// can be unsigned integer as the block_size is at most INT_MAX ?
+    std::unique_ptr<unsigned long long[]> comp_sizes_p(new unsigned long long[nr_of_blocks + 1]);
+    unsigned long long* comp_sizes = comp_sizes_p.get();
 
-    std::unique_ptr<unsigned long long[]> compBatchSizesP(new unsigned long long[nrOfThreads]);
-    unsigned long long* compBatchSizes = compBatchSizesP.get();
+    std::unique_ptr<unsigned long long[]> comp_batch_sizes_p(new unsigned long long[nr_of_threads]);
+    unsigned long long* comp_batch_sizes = comp_batch_sizes_p.get();
 
-    std::unique_ptr<unsigned long long[]> blockHashesP;
+	// each block has it's own hash calculated
+    std::unique_ptr<unsigned long long[]> block_hashes_p;
 
     if (hash)
     {
-      blockHashesP = std::unique_ptr<unsigned long long[]>(new unsigned long long[nrOfBlocks]);
+      block_hashes_p = std::unique_ptr<unsigned long long[]>(new unsigned long long[nr_of_blocks]);
     }
 
-    unsigned long long* blockHashes = blockHashesP.get();
+    unsigned long long* block_hashes = block_hashes_p.get();
 
-    // We need nrOfBlocks buffers
-    unsigned int compressionAlgo;
+    // We need nr_of_blocks buffers
+    unsigned int compression_algorithm;
 
-    std::unique_ptr<unsigned char[]> calcBufferP(new unsigned char[bufSize]);
-    unsigned char* calcBuffer = calcBufferP.get();
+    std::unique_ptr<unsigned char[]> calc_buffer_p(new unsigned char[buf_size]);
+    unsigned char* calc_buffer = calc_buffer_p.get();
 
-#pragma omp parallel num_threads(nrOfThreads) shared(compSizes,nrOfThreads,compBatchSizes)
+#pragma omp parallel num_threads(nr_of_threads) shared(comp_sizes,nr_of_threads,comp_batch_sizes)
     {
 #pragma omp for schedule(static, 1) nowait
-      for (int blockBatch = 0; blockBatch < (nrOfThreads - 1); blockBatch++) // all but last batch
+      for (int block_batch = 0; block_batch < (nr_of_threads - 1); block_batch++) // all but last batch
       {
-        CompAlgo compAlgo;
-        const float blockOffset = blockBatch * blocksPerThread;
-        const int blockNr = static_cast<int>(0.00001 + blockOffset);
-        const int nextblockNr = static_cast<int>(blocksPerThread + 0.00001 + blockOffset);
+        CompAlgo comp_algo;
+        const double block_offset = block_batch * blocks_per_thread;
+        const int block_nr = static_cast<int>(DOUBLE_DELTA + block_offset);
+        const int next_block_nr = static_cast<int>(blocks_per_thread + DOUBLE_DELTA + block_offset);
 
-        unsigned char* threadBuf = calcBuffer + maxCompressSize * blockNr; // buffer for compression results of current thread
+		// buffer for compression results of current thread
+        unsigned char* thread_buf = calc_buffer + static_cast<unsigned long long>(max_compress_size) * block_nr;
 
         // inner loop is executed by current thread
-        unsigned long long bufPos = 0;
-        for (int block = blockNr; block < nextblockNr; block++)
+        unsigned long long buf_pos = 0;
+        for (int block = block_nr; block < next_block_nr; block++)
         {
-          const int compSize = this->compressor->Compress(reinterpret_cast<char*>(threadBuf + bufPos), maxCompressSize,
-            reinterpret_cast<char*>(&blobSource[block * blockSize]), blockSize, compAlgo);
-          compSizes[block] = static_cast<unsigned long long>(compSize);
+			// error when > 4 GB !
+			const int comp_size = this->compressor->Compress(reinterpret_cast<char*>(thread_buf + buf_pos), max_compress_size,
+				reinterpret_cast<char*>(&blob_source[block_size * static_cast<unsigned long long>(block)]), block_size, comp_algo);
 
-          // Hash compression result
-          if (hash)
-          {
-            blockHashes[block] = XXH64(threadBuf + bufPos, compSize, FST_HASH_SEED);
-          }
+			// this cast is a waste of format space at it is at most INT_MAX!
+			comp_sizes[block] = static_cast<unsigned long long>(comp_size);
 
-          bufPos += compSize;
-        }
+			// Hash compression result
+			if (hash)
+			{
+				block_hashes[block] = XXH64(thread_buf + buf_pos, comp_size, FST_HASH_SEED);
+			}
 
-        compBatchSizes[blockBatch] = bufPos;
+			buf_pos += comp_size;
+		}
+
+		comp_batch_sizes[block_batch] = buf_pos;
       }
 
-// last thread might have a smaller task
+// last thread might have a smaller task (and is executed in parallel with above loop)
 #pragma omp single
       {
-        CompAlgo compAlgo;
-        const int block_nr = static_cast<int>(0.00001 + (nrOfThreads - 1) * blocksPerThread);
-        const int nextblock_nr = static_cast<int>(0.00001 + (nrOfThreads * blocksPerThread)) - 1; // exclude last block
+        CompAlgo comp_algo;
 
-        unsigned char* threadBuf = calcBuffer + maxCompressSize * block_nr; // buffer for compression results of current thread
+		const double block_offset = (nr_of_threads - 1) * blocks_per_thread;
+      	const int block_nr = static_cast<int>(DOUBLE_DELTA + block_offset);
+        const int next_block_nr = static_cast<int>(blocks_per_thread + DOUBLE_DELTA + block_offset) - 1; // exclude last block
 
-        // inner loop is executed by current thread
-        unsigned long long bufPos = 0;
-        for (int block = block_nr; block < nextblock_nr; block++)
+		// buffer for compression results of current thread
+		unsigned char* thread_buf = calc_buffer + static_cast<unsigned long long>(max_compress_size) * block_nr;
+
+        // might not be executed if blocks_per_thread equals 1
+        unsigned long long buf_pos = 0;
+        for (int block = block_nr; block < next_block_nr; block++)
         {
-          const int compSize = this->compressor->Compress(reinterpret_cast<char*>(threadBuf + bufPos), maxCompressSize,
-            reinterpret_cast<char*>(&blobSource[block * blockSize]), blockSize, compAlgo);
-          compSizes[block] = static_cast<unsigned long long>(compSize);
+          const int comp_size = this->compressor->Compress(reinterpret_cast<char*>(&thread_buf[buf_pos]), max_compress_size,
+            reinterpret_cast<char*>(&blob_source[block_size * static_cast<unsigned long long>(block)]), block_size, comp_algo);
+          comp_sizes[block] = static_cast<unsigned long long>(comp_size);
 
           // Hash compression result
           if (hash)
           {
-            blockHashes[block] = XXH64(threadBuf + bufPos, compSize, FST_HASH_SEED);
+            block_hashes[block] = XXH64(thread_buf + buf_pos, comp_size, FST_HASH_SEED);
           }
 
-          bufPos += compSize;
+          buf_pos += comp_size;
         }
 
-        // last block
-        const int compSize = this->compressor->Compress(reinterpret_cast<char*>(threadBuf + bufPos), maxCompressSize,
-          reinterpret_cast<char*>(&blobSource[nextblock_nr * blockSize]), lastBlockSize, compAlgo);
-        compSizes[nextblock_nr] = static_cast<unsigned long long>(compSize);
+        // last block might be smaller
+        const int comp_size = this->compressor->Compress(reinterpret_cast<char*>(&thread_buf[buf_pos]), max_compress_size,
+          reinterpret_cast<char*>(&blob_source[block_size * static_cast<unsigned long long>(next_block_nr)]), last_block_size, comp_algo);
+        comp_sizes[next_block_nr] = static_cast<unsigned long long>(comp_size);
 
-        compBatchSizes[nrOfThreads - 1] = bufPos + compSize;
-        compressionAlgo = static_cast<unsigned int>(compAlgo);
+        comp_batch_sizes[nr_of_threads - 1] = buf_pos + comp_size;
+        compression_algorithm = static_cast<unsigned int>(comp_algo);
 
         // Hash compression result
         if (hash)
         {
-          blockHashes[nextblock_nr] = XXH64(threadBuf + bufPos, compSize, FST_HASH_SEED);
+          block_hashes[next_block_nr] = XXH64(thread_buf + buf_pos, comp_size, FST_HASH_SEED);
         }
       }
     } // end parallel region and join all threads
@@ -231,18 +251,18 @@ public:
     // calculate hask of block hashes
     if (hash)
     {
-      allBlockHash = XXH64(blockHashes, nrOfBlocks * 8, FST_HASH_SEED);
+      allBlockHash = XXH64(block_hashes, nr_of_blocks * 8, FST_HASH_SEED);
     }
 
-    unsigned long long totCompSize = 0;
-    for (int blockBatch = 0; blockBatch < nrOfThreads; blockBatch++)
+    unsigned long long tot_comp_size = 0;
+    for (int block_batch = 0; block_batch < nr_of_threads; block_batch++)
     {
-      totCompSize += compBatchSizes[blockBatch];
+      tot_comp_size += comp_batch_sizes[block_batch];
     }
 
     // In memory compression format:
     // Size                 | Type               | Description
-    // 4                    | insigned int       | header hash
+    // 4                    | unsigned int       | header hash
     // 4                    | unsigned int       | blockSize
     // 4                    | unsigned int       | version
     // 4                    | unsigned int       | COMPRESSION_ALGORITHM and upper bit isHashed
@@ -251,8 +271,8 @@ public:
     // 8 * (nrOfBlocks + 1) | unsigned long long | block offset
     //                      | unsigned char      | compressed data
 
-    const unsigned long long headerSize = 8 * (nrOfBlocks + 5);
-    IBlobContainer* blobContainer = typeFactory->CreateBlobContainer(headerSize + totCompSize);
+    const unsigned long long headerSize = 8 * (nr_of_blocks + 5);
+    IBlobContainer* blobContainer = typeFactory->CreateBlobContainer(headerSize + tot_comp_size);
     unsigned char* blobData = blobContainer->Data();
 
     unsigned int* headerHash = reinterpret_cast<unsigned int*>(blobData);
@@ -263,41 +283,41 @@ public:
     unsigned long long* hashResult = reinterpret_cast<unsigned long long*>(blobData + 24);
     unsigned long long* blockOffsets = reinterpret_cast<unsigned long long*>(blobData + 32);
 
-    *pBlockSize = blockSize;
-    *version = 1;
-    *algo = compressionAlgo | ((1 << 31) * hash); // upper bit signals isHashed
-    *vecLength = blobLength;
+    *pBlockSize = block_size;
+    *version = FST_COMPRESS_VERSION;
+    *algo = compression_algorithm | ((1 << 31) * hash); // upper bit signals isHashed
+    *vecLength = blob_length;
     *hashResult = allBlockHash;
 
     // calculate offsets for memcpy
     unsigned long long dataOffset = headerSize;
 
-    std::unique_ptr<unsigned long long[]> dataOffsetsP(new unsigned long long[nrOfThreads]);
+    std::unique_ptr<unsigned long long[]> dataOffsetsP(new unsigned long long[nr_of_threads]);
     unsigned long long* dataOffsets = dataOffsetsP.get();
 
-    for (int blockBatch = 0; blockBatch < nrOfThreads; blockBatch++)
+    for (int blockBatch = 0; blockBatch < nr_of_threads; blockBatch++)
     {
       dataOffsets[blockBatch] = dataOffset;
-      dataOffset += compBatchSizes[blockBatch];
+      dataOffset += comp_batch_sizes[blockBatch];
     }
 
     // multi-threaded memcpy
 #pragma omp parallel for schedule(static, 1)
-    for (int blockBatch = 0; blockBatch < nrOfThreads; blockBatch++)
+    for (int blockBatch = 0; blockBatch < nr_of_threads; blockBatch++)
     {
-      const float blockOffset = blockBatch * blocksPerThread;
-      const int blockNr = static_cast<int>(0.00001 + blockOffset);
-      unsigned char* threadBuf = calcBuffer + maxCompressSize * blockNr; // buffer for compression results of current thread
-      std::memcpy(blobData + dataOffsets[blockBatch], threadBuf, compBatchSizes[blockBatch]);
+      const double blockOffset = blockBatch * blocks_per_thread;
+      const int blockNr = static_cast<int>(DOUBLE_DELTA + blockOffset);
+      unsigned char* threadBuf = calc_buffer + static_cast<unsigned long long>(max_compress_size) * blockNr; // buffer for compression results of current thread
+      std::memcpy(blobData + dataOffsets[blockBatch], threadBuf, comp_batch_sizes[blockBatch]);
     } // end parallel region
 
     unsigned long long blockOffset = headerSize;
-    for (int block = 0; block < nrOfBlocks; block++)
+    for (unsigned int block = 0; block != nr_of_blocks; block++)
     {
       blockOffsets[block] = blockOffset;
-      blockOffset += compSizes[block];
+      blockOffset += comp_sizes[block];
     }
-    blockOffsets[nrOfBlocks] = blockOffset;
+    blockOffsets[nr_of_blocks] = blockOffset;
 
     *headerHash = XXH32(&blobData[4], headerSize - 4, FST_HASH_SEED); // header hash
 
@@ -331,7 +351,9 @@ public:
 
     unsigned int* headerHash = reinterpret_cast<unsigned int*>(blobSource);
     unsigned int* blockSize = reinterpret_cast<unsigned int*>(blobSource + 4);
-    //unsigned int* version            = reinterpret_cast<unsigned int*>(blobSource + 8);
+
+	unsigned int* version = reinterpret_cast<unsigned int*>(blobSource + 8);
+	
     unsigned int* algo = reinterpret_cast<unsigned int*>(blobSource + 12);
     unsigned long long* vecLength = reinterpret_cast<unsigned long long*>(blobSource + 16);
     unsigned long long* hashResult = reinterpret_cast<unsigned long long*>(blobSource + 24);
@@ -353,30 +375,33 @@ public:
 
     const unsigned int headHash = XXH32(&blobSource[4], headerSize - 4, FST_HASH_SEED); // header hash
 
+	// header hash check
     if (*headerHash != headHash)
     {
       throw(std::runtime_error(FSTERROR_COMP_HEADER));
     }
 
+    // version check
+	if (*version > FST_COMPRESS_VERSION)
+	{
+		throw(std::runtime_error(FSTERROR_COMP_FUTURE_VERSION));
+	}
 
-    // Version checks here
+	// Source vector has correct length
+	if (blockOffsets[nrOfBlocks] != blobLength)
+	{
+		throw(std::runtime_error(FSTERROR_COMP_SIZE));
+	}
 
     // Create result blob
-    IBlobContainer* blobContainer = typeFactory->CreateBlobContainer(*vecLength); // create result blob
-    unsigned char* blobData = blobContainer->Data();
-
-    // Source vector has correct length
-    if (blockOffsets[nrOfBlocks] != blobLength)
-    {
-      delete blobContainer;
-      throw(std::runtime_error(FSTERROR_COMP_SIZE));
-    }
+	IBlobContainer* blob_container = typeFactory->CreateBlobContainer(*vecLength);
+    unsigned char* blob_data = blob_container->Data();
 
     // Determine required number of threads
     nrOfThreads = std::min(nrOfBlocks, nrOfThreads);
 
     // Determine number of blocks per (thread) batch
-    const float batchFactor = static_cast<float>(nrOfBlocks) / nrOfThreads;
+    const double batchFactor = static_cast<double>(nrOfBlocks) / nrOfThreads;
 
     bool error = false;
 
@@ -390,8 +415,8 @@ public:
 #pragma omp for schedule(static, 1) nowait
         for (int batch = 0; batch < (nrOfThreads - 1); batch++) // all but last batch
         {
-          const int fromBlock = static_cast<int>(batch * batchFactor + 0.000001); // start block
-          const int toBlock = static_cast<int>((batch + 1) * batchFactor + 0.000001); // end block
+          const int fromBlock = static_cast<int>(batch * batchFactor + DOUBLE_DELTA); // start block
+          const int toBlock = static_cast<int>((batch + 1) * batchFactor + DOUBLE_DELTA); // end block
 
           // iterate block range
           for (int block = fromBlock; block < toBlock; block++)
@@ -405,8 +430,8 @@ public:
 
 #pragma omp single
         {
-          const int fromBlock = static_cast<int>((nrOfThreads - 1) * batchFactor + 0.000001); // start block
-          const int toBlock = static_cast<int>(nrOfThreads * batchFactor + 0.000001); // end block
+          const int fromBlock = static_cast<int>((nrOfThreads - 1) * batchFactor + DOUBLE_DELTA); // start block
+          const int toBlock = static_cast<int>(nrOfThreads * batchFactor + DOUBLE_DELTA); // end block
 
           // iterate block range with full blocks
           for (int block = fromBlock; block < (toBlock - 1); block++)
@@ -429,7 +454,7 @@ public:
 
       if (totHashes != *hashResult)
       {
-        delete blobContainer;
+        delete blob_container;
         throw(std::runtime_error(FSTERROR_COMP_DATA_HASH));
       }
     }
@@ -439,8 +464,8 @@ public:
 #pragma omp for schedule(static, 1) nowait
       for (int batch = 0; batch < (nrOfThreads - 1); batch++) // all but last batch
       {
-        const int fromBlock = static_cast<int>(batch * batchFactor + 0.000001); // start block
-        const int toBlock = static_cast<int>((batch + 1) * batchFactor + 0.000001); // end block
+        const int fromBlock = static_cast<int>(batch * batchFactor + DOUBLE_DELTA); // start block
+        const int toBlock = static_cast<int>((batch + 1) * batchFactor + DOUBLE_DELTA); // end block
 
         // iterate block range
         for (int block = fromBlock; block < toBlock; block++)
@@ -448,7 +473,7 @@ public:
           const unsigned long long blockStart = blockOffsets[block];
           const unsigned long long blockEnd = blockOffsets[block + 1];
 
-          const unsigned int errorCode = decompressor.Decompress(algorithm, reinterpret_cast<char*>(blobData) + *blockSize * block,
+          const unsigned int errorCode = decompressor.Decompress(algorithm, reinterpret_cast<char*>(blob_data) + *blockSize * block,
             *blockSize, reinterpret_cast<const char*>(blobSource + blockStart), blockEnd - blockStart);
 
           if (errorCode != 0)
@@ -460,15 +485,15 @@ public:
 
 #pragma omp single
       {
-        const int fromBlock = static_cast<int>((nrOfThreads - 1) * batchFactor + 0.000001); // start block
-        const int toBlock = static_cast<int>(nrOfThreads * batchFactor + 0.000001); // end block
+        const int fromBlock = static_cast<int>((nrOfThreads - 1) * batchFactor + DOUBLE_DELTA); // start block
+        const int toBlock = static_cast<int>(nrOfThreads * batchFactor + DOUBLE_DELTA); // end block
 
         // iterate block range with full blocks
         for (int block = fromBlock; block < (toBlock - 1); block++)
         {
           const unsigned long long blockStart = blockOffsets[block];
           const unsigned long long blockEnd = blockOffsets[block + 1];
-          const unsigned int errorCode = decompressor.Decompress(algorithm, reinterpret_cast<char*>(blobData) + *blockSize * block, *blockSize,
+          const unsigned int errorCode = decompressor.Decompress(algorithm, reinterpret_cast<char*>(blob_data) + *blockSize * block, *blockSize,
             reinterpret_cast<const char*>(blobSource + blockStart), blockEnd - blockStart);
 
           if (errorCode != 0)
@@ -481,7 +506,7 @@ public:
         const int lastBlockSize = 1 + (*vecLength - 1) % *blockSize;
         const unsigned long long blockStart = blockOffsets[toBlock - 1];
         const unsigned long long blockEnd = blockOffsets[toBlock];
-        const unsigned int errorCode = decompressor.Decompress(algorithm, reinterpret_cast<char*>(blobData) + *blockSize * (toBlock - 1),
+        const unsigned int errorCode = decompressor.Decompress(algorithm, reinterpret_cast<char*>(blob_data) + *blockSize * (toBlock - 1),
           lastBlockSize, reinterpret_cast<const char*>(blobSource + blockStart), blockEnd - blockStart);
 
         if (errorCode != 0)
@@ -494,11 +519,11 @@ public:
 
     if (error)
     {
-      delete blobContainer;
-      throw(std::runtime_error(FSTERROR_COMP_STREAM));
+		delete blob_container;
+		throw(std::runtime_error(FSTERROR_COMP_STREAM));
     }
 
-    return blobContainer;
+    return blob_container;
   }
 };
 
